<?php
	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	//require_once( dirname(__FILE__) . "/app.php.inc" );
	require_once( dirname(__FILE__) . "/manager.util.php.inc" );
	require_once( dirname(__FILE__) . "/manager.base.php.inc" );

	/* --------------------------------------------------------------------
	 * エントリヘッダ入力項目クラス。header
	 * 
	 */ 
	class HeaderEntryItem extends BaseEntryItem
	{
		var $publishControl;	// 公開方法
		
		/* ----------------------------------------------------------------
		 * コンストラクタ。
		 */
		function HeaderEntryItem( $entryName, $entryId, $publishControl )
		{
			// ベースコンストラクタを呼び出さない。
			//parent::BaseEntryItem( $xmlSchemaData, $entryName, $entryId  );
			
			// ベースクラスのプロパティを設定
			$this->dataName = "header"; //$xmlSchemaData->get_attribute('name');
			$this->dataType = "header"; //$xmlSchemaData->get_attribute('type');
			$this->dataCaption = "公開設定／作成者"; //file2code( $xmlSchemaData->get_attribute('caption') );
			
			$this->entryName = $entryName;
			$this->entryId = $entryId;
			
			$this->publishControl = $publishControl;
			
	   	}
		
		
		/* ----------------------------------------------------------------
		 * 当項目のXMLノードを新規作成する。
		 * $xmlDocParentEntry : エントリのDOMオブジェクト。
		 * return : 新規XMLノード。
		 */
		function createNew( $xmlDocParentEntry )
		{
   			//$newNode = parent::createNew( $xmlDocParentEntry );
			
			cmsd_abort_proc( 'このメソッドは呼び出し不可' );
			
			return $newNode;
		}
		
		/* ----------------------------------------------------------------
		 * 与えた日時に対する週を返す。
		 * return : 週を表す3文字の文字列。
		 */
		function getWeekday( $year, $month, $day, $hour, $minute, $second )
		{
			$arrWeekday = array( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
			$iWeekday = date('w', mktime( $hour, $minute, $second, $month, $day, $year ) );
			$sWeekday = $arrWeekday[ $iWeekday ];
			
			return $sWeekday;
		}
		
		/* ----------------------------------------------------------------
		 * 与えた日時を１つの文字列に結合する。
		 * return : 日時を表す一続きの文字列。
		 */
		function getDateString( $dataType, $year, $month, $day, $hour, $minute, $second )
		{
			// 年月日時分秒
			$sYear = sprintf( "%04d", $year );
			$sMonth = sprintf( "%02d", $month );
			$sDay = sprintf( "%02d", $day );
			$sHour = sprintf( "%02d", $hour );
			$sMinute = sprintf( "%02d", $minute );
			$sSecond = sprintf( "%02d", $second );

			switch( $dataType )
			{
				case 'condition' :
					$strDate = $sYear . $sMonth . $sDay . $sHour . $sMinute;
					break;
				
				case 'datetime' :
					$strDate = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
					break;
				
				case 'date' :
					$strDate = $sYear . '-' . $sMonth . '-' . $sDay;
					break;
				
				case 'time' :
					$strDate = $sHour . ':' . $sMinute;
					break;
				
				case 'year' :
					$strDate = $sYear;
					break;
				
				case 'month' :
					$strDate = $sYear . '-' . $sMonth;
					break;
				
			}
			
			return $strDate;
		}
		
		/* ----------------------------------------------------------------
		 * 日時文字列を分解する。
		 * return : 値の配列
		 */
		function parseDateString( $strDate )
		{
			$strDate = str_replace( array( ':',' ','-' ), array( '','','' ), $strDate );
			
			// YYYYMMDDHHmm
			$sYear = substr( $strDate, 0, 4 );
			$sMonth = substr( $strDate, 4, 2 );
			$sDay = substr( $strDate, 6, 2 );
			$sHour = substr( $strDate, 8, 2 );
			$sMinute = substr( $strDate, 10, 2 );
			$sSecond = substr( $strDate, 12, 2 );

			/*
			// YYYY-MM-DD HH:mm
			$sYear = substr( $strDate, 0, 4 );
			$sMonth = substr( $strDate, 5, 2 );
			$sDay = substr( $strDate, 8, 2 );
			$sHour = substr( $strDate, 11, 2 );
			$sMinute = substr( $strDate, 14, 2 );
			$sSecond = substr( $strDate, 17, 2 );
			*/
			
			return array( $sYear, $sMonth, $sDay, $sHour, $sMinute, $sSecond );
		
		}
		
		/* ----------------------------------------------------------------
		 * POSTリクエストから日時情報を取得する。
		 * return : 日時情報の配列。エラーの場合はFalseを返す。
		 */
		function getRequestYMDHM( $dataType, $dataName )
		{
			// year
			if ( in_array( $dataType, array( 'entrydate', 'datetime', 'date', 'year', 'month' ) ) )
			{
				$year = $_POST[ $dataName . '_year' ];
				
				// 範囲チェック
				$iYear = intval( $year ); 
				if ( $iYear < 0 or $iYear > 9999 )
				{
					$this->errorMessage = sprintf( "%s(年) が 0-9999の範囲を超えました", $this->dataCaption );
					return false;
				}
				
			}
			
			// month
			if ( in_array( $dataType, array( 'entrydate', 'datetime', 'date', 'month' ) ) )
			{
				$month = $_POST[ $dataName . '_month' ];
				
				// 範囲チェック
				$iMonth = intval( $month ); 
				if ( $iMonth < 0 or $iMonth > 12 )
				{
					$this->errorMessage = sprintf( "%s(月) が 0-12の範囲を超えました", $this->dataCaption );
					return false;
				}
				
			}

			// day
			if ( in_array( $dataType, array( 'entrydate', 'datetime', 'date' ) ) )
			{
				$day = $_POST[ $dataName . '_day' ];
				
				// 範囲チェック
				$iDay = intval( $day ); 
				if ( $iDay < 0 or $iDay > 31 )
				{
					$this->errorMessage = sprintf( "%s(日) が 0-31の範囲を超えました", $this->dataCaption );
					return false;
				}
			}
			
			// hour
			if ( in_array( $dataType, array( 'entrydate', 'datetime', 'time' ) ) )
			{
				$hour = $_POST[ $dataName . '_hour' ];
				
				// 範囲チェック
				$iHour = intval( $hour ); 
				if ( $iHour < 0 or $iHour > 23 )
				{
					$this->errorMessage = sprintf( "%s(時) が 0-24の範囲を超えました", $this->dataCaption );
					return false;
				}
			}

			// minute
			if ( in_array( $dataType, array( 'entrydate', 'datetime', 'time' ) ) )
			{
				$minute = $_POST[ $dataName . '_minute' ];
				
				// 範囲チェック
				$iMinute = intval( $minute ); 
				if ( $iMinute < 0 or $iMinute > 59 )
				{
					$this->errorMessage = sprintf( "%s(時) が 0-59の範囲を超えました", $this->dataCaption );
					return false;
				}
			}
			
			return array( $year, $month, $day, $hour, $minute );
		
		}
		
		
		/* ----------------------------------------------------------------
		 * 入力内容でエントリ項目を更新する。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function update( $xpathEntry, $currentEntry )
		{
			// エントリXMLを取得。
			/// entry要素自身を取得して更新する。
			$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry );
			$xmlEntryData = $xmlEntryDataArr->nodeset[0];
			
			// 公開フラグ（画面上では「非公開」チェック）
			$visible = getPOSTParam( 'cmsd_visible' ); //$_POST[ 'cmsd_visible' ];
			$xmlEntryData->set_attribute('visible', ( $visible ? 'False' : 'True' ) );	// 画面上では「非公開」チェックなので、値があれば"False"になる。
			
			// 公開開始フラグ
			$startDateActive = getPOSTParam( 'cmsd_sd_active' ); //$_POST[ 'cmsd_sd_active' ];
			if ( $startDateActive )
			{
				// 公開開始フラグがONの時は、公開開始時刻も取得する。
				// 公開開始時刻
				$cmsd_sd_ymdhm = $this->getRequestYMDHM( 'datetime', 'cmsd_sd' );
				if ( ! $cmsd_sd_ymdhm ) 
				{
					return false;	// 入力内容エラー時は以降の処理をキャンセルしてreturn。
				}
				list( $year, $month, $day, $hour, $minute ) = $cmsd_sd_ymdhm;
				$strStartDate = $this->getDateString( 'condition', $year, $month, $day, $hour, $minute, 0 );
			}
			else
			{
				// 公開開始フラグがOFFの時は、0埋めで設定する。
				$strStartDate = $this->getDateString( 'condition', 0, 0, 0, 0, 0, 0 );
			}
			
			// 公開終了フラグ
			$endDateActive = getPOSTParam( 'cmsd_ed_active' ); //$_POST[ 'cmsd_ed_active' ];
			if ( $endDateActive )
			{
				// 公開終了フラグがONの時は、公開終了時刻も取得する。
				// 公開終了時刻
				$cmsd_ed_ymdhm = $this->getRequestYMDHM( 'datetime', 'cmsd_ed' );
				if ( ! $cmsd_ed_ymdhm )
				{
					return false;	// 入力内容エラー時は以降の処理をキャンセルしてreturn。
				}
				list( $year, $month, $day, $hour, $minute ) = $cmsd_ed_ymdhm;
				$strEndDate = $this->getDateString( 'condition', $year, $month, $day, $hour, $minute, 0 );
			}
			else
			{
				// 公開終了フラグがOFFの時は、9埋めで設定する。
				$strEndDate = $this->getDateString( 'condition', 9999, 99, 99, 99, 99, 0 );
			}
			
			// 開始～終了期間チェック
			if ( $strStartDate > $strEndDate )
			{
				$this->errorMessage = '終了時刻を開始時刻より早く設定することは出来ません。';
				return false;
			}
			else
			{
				$xmlEntryData->set_attribute( 'enddate', $strEndDate );
				$xmlEntryData->set_attribute( 'startdate', $strStartDate );
			}
			
			// 作成者チェック [2008.04 ユーザー権限]
			$author = getPOSTParam( 'cmsd_author' );
			$authedUser = LoginManager::GetAuthedUser();
			
			if ( strlen( $author ) <= 0 ) 
			{
				// データが空ならば変更なし（何もしない）※Admin以外のユーザーでは、編集画面の作成者欄は固定になっており、そもそもform項目になっていない。
			}
			elseif ( ! SiteUser::UserExists( $author ) )
			{
				// ユーザーリストに存在しなければエラー
				$this->errorMessage = '作成者名に、存在しないユーザーIDが指定されています';
				return false;
			}
			elseif ( ! ( $authedUser->isAdministrator() || $autheduser->getUserId() == $author ) )
			{
				// ログインユーザーがAdminか、ログインユーザー名と作成者名が同一ならばOK、それ以外は「不正な作成者ID」エラー。
			}
			else
			{
				// チェックを通ったので値を設定する。
				$xmlEntryData->set_attribute( 'author', $author );
			}
			
			return true;
			
		}
		
		/* ----------------------------------------------------------------
		 * フォーム項目を描画するHTMLを返す。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function renderFormItemXML( $xpathEntry, $currentEntry, &$xmlOutputEntryData )
		{
			// 現在の階層を取得（entry要素を取得）。
			$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry );
   			$xmlEntryData = $xmlEntryDataArr->nodeset[0];
			
			// データの取り出し
			// 公開フラグ
			$visible = ( $xmlEntryData->get_attribute('visible') == 'False' ? 'False' : 'True' );	// 公開フラグが無かった頃のデータとの互換性の為、"False"と比較する。
			$xmlOutputEntryData->set_attribute('visible', $visible );
			
			// 公開開始日時
			$startDate = $xmlEntryData->get_attribute('startdate');
			if ( $startDate == $this->getDateString('condition', 0, 0, 0, 0, 0, 0 ) or strlen( $startDate ) == 0 )
			{
				$xmlOutputEntryData->set_attribute('sd_active', 'False' );
				$xmlOutputEntryData->set_attribute('sd_year', sprintf( "%04d", date('Y') ) );
				$xmlOutputEntryData->set_attribute('sd_month',sprintf( "%02d", date('m') ) );
				$xmlOutputEntryData->set_attribute('sd_day', sprintf( "%02d", date('d') ) );
				$xmlOutputEntryData->set_attribute('sd_hour', sprintf( "%02d", date('H') ) );
				$xmlOutputEntryData->set_attribute('sd_minute', sprintf( "%02d", date('i') ) );
			}
			else
			{
				$xmlOutputEntryData->set_attribute('sd_active', 'True' );
				list( $year, $month, $day, $hour, $minute, $second ) = $this->parseDateString( $startDate );
				$xmlOutputEntryData->set_attribute('sd_year', $year );
				$xmlOutputEntryData->set_attribute('sd_month', $month );
				$xmlOutputEntryData->set_attribute('sd_day', $day );
				$xmlOutputEntryData->set_attribute('sd_hour', $hour );
				$xmlOutputEntryData->set_attribute('sd_minute', $minute );
			}
			
			// 公開終了日時
			$endDate = $xmlEntryData->get_attribute('enddate');
			if ( $endDate == $this->getDateString( 'condition', 9999, 99, 99, 99, 99, 0 ) or strlen( $endDate ) == 0 )
			{
				$xmlOutputEntryData->set_attribute('ed_active', 'False' );
				$xmlOutputEntryData->set_attribute('ed_year', sprintf( "%04d", date('Y') ) );
				$xmlOutputEntryData->set_attribute('ed_month',sprintf( "%02d", date('m') ) );
				$xmlOutputEntryData->set_attribute('ed_day', sprintf( "%02d", date('d') ) );
				$xmlOutputEntryData->set_attribute('ed_hour', sprintf( "%02d", date('H') ) );
				$xmlOutputEntryData->set_attribute('ed_minute', sprintf( "%02d", date('i') ) );
			}
			else
			{
				$xmlOutputEntryData->set_attribute('ed_active', 'True' );
				list( $year, $month, $day, $hour, $minute, $second ) = $this->parseDateString( $endDate );
				$xmlOutputEntryData->set_attribute('ed_year', $year );
				$xmlOutputEntryData->set_attribute('ed_month', $month );
				$xmlOutputEntryData->set_attribute('ed_day', $day );
				$xmlOutputEntryData->set_attribute('ed_hour', $hour );
				$xmlOutputEntryData->set_attribute('ed_minute', $minute );
			}
			
			// 公開方法
			$xmlOutputEntryData->set_attribute('publishcontrol', $this->publishControl );
			
			// ユーザーID [2008.04 ユーザー権限]
			$userId = $xmlEntryData->get_attribute( 'author' );
			$xmlOutputEntryData->set_attribute( 'userid', $userId );
			
		}
		
	
	}

?>