<?php
	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	require_once( dirname(__FILE__) . "/xmlconfig.php.inc" );
	
	// グローバル変数 ------------------------------------------------
	
	// XSLTの出力エンコーディングを無効にするか？
	$gXmlutil_DisableXsltEncoding = false;
	
	function mb_chr($num){
  		return ($num < 256) ? chr($num) : mb_chr($num / 256).chr($num % 256);
	}
	 
	// 同形異値の文字コードを置換する処理。eucJP-win対応の副作用回避用。→2015.08.27 CMSDの内部処理UTF-8化対応により、不要な処理となった為、基本的には何もしないコードとなった
	function utf8_replace_irregular_chars( $str )
	{
		if ( gCMSDEnableConvertingEncoding() ) {
			$str = str_replace( mb_chr( 0xefbd9e ), mb_chr( 0xe3809c ), $str );	// 波形文字を U+FF5E から U+301C へ置換。
			//$str = str_replace( "\xef\xbd\x9e", "\xe3\x80\x9c", $str );
		}
		return $str;
	}	
	
	/* ---------------------------------------------------------------
	 * XSLTライブラリのラッパークラス。
	 */
	class ADDom
	{
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * XSLT変換を実行する。
		 * &$xmlDoc : DOMオブジェクト。
		 * $filePath : ファイルのパス。
		 * $outputEncoding : 出力用のエンコーディング。
		 */
		function SaveXmlToFile( &$xmlDoc, $filePath, $outputEncoding = 'UTF-8' )
		{
			$strXml = $xmlDoc->dump_mem( true, $outputEncoding );
			$fp = fopen( $filePath, 'w' );
			set_file_buffer($fp, 0);
			flock( $fp, LOCK_EX );
			rewind($fp);
			
			fwrite( $fp, $strXml );
			
			flock( $fp, LOCK_UN );
			fclose( $fp );
			
			//$xmlDocEntryIndex->dump_file( App::createEntryFilePath( $entryName, 'index' ), false, true );
		}

	}
	
	function ADXslt_error_handler( $handler, $errno, $level, $info )
	{
		$errorMessage = "<h3>CMS Designer : Xsl Translation " . $info['msgtype'] . "</h3>\n" .
						"<dl>\n" .
						"<dt>[message]</dt><dd>" . $info['msg'] . " (code: " . $info['code'] . ")</dd>\n" .
						"<dt>[XSL]</dt><dd>" . $info['URI'] . "</dd>\n" .
						"<dt>[line]</dt><dd>" . $info['line'] . "</dd>\n" .
						"<dt>[module]</dt><dd>" . $info['module'] . "</dd>\n" .
						"</dl>";
		
		cmsd_abort_proc( $errorMessage, false );
	}
	
	function ADXslt_error_handler2( $errno, $errstr )
	{
		$errorMessage = "<h3>CMS Designer : Xsl Translation error" . "</h3>\n" .
						"<dl>\n" .
						"<dt>[message]</dt><dd>" . $errstr . "</dd>\n";
		print( $errorMessage );
	}
	
	/* ---------------------------------------------------------------
	* XSLTライブラリのラッパークラス。
	*/
	class ADXslt
	{
	
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * Xsltライブラリ設定を行う
		*/
		
		function setXsltLibNo( $newValue )
		{
			global $gXsltLib;
			$gXsltLib = $newValue;
		}
		
		function getDisableXsltEncoding()
		{
			global $gXmlutil_DisableXsltEncoding;
			return $gXmlutil_DisableXsltEncoding;
		}
		
		function setDisableXsltEncoding( $newValue )
		{
			global $gXmlutil_DisableXsltEncoding;
			$gXmlutil_DisableXsltEncoding = $newValue;
		}
		
	
		function getXsltLibNo()
		{
			global $gXsltLib;
		
			$xsltLib = 10;	// 初期値はSabloton
		
			if ( $gXsltLib == 0 )
			{
				if ( function_exists( 'xslt_create' ) && function_exists( 'xslt_process' ) && function_exists( 'xslt_free' ) ) 
				{
					$xsltLib = 10;	// Sabloton
				}
				elseif ( function_exists( 'domxml_xslt_stylesheet_file' ) )
				{
					$xsltLib = 20;	// domxml
				}
				else
				{
					$xsltLib = 99;	// 該当なし
				}
			}
			else
			{
				$xsltLib = $gXsltLib;
			}
			
			return $xsltLib;
		
		}
	
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * XSLT変換を実行する。
		 * &$xmlDoc : 変換対象のDOMオブジェクト。
		 * $strXsl : XSLファイルのパス。
		 * $outputEncoding : 出力用のエンコーディング。通常はXSLT側で指定したものが使われるが、環境によっては無視されるので直接指定する。
		 * $isXslString : $strXslが、ファイルパスではなくXSLそのものの場合。
		*/
		function process( &$xmlDoc, $strXsl, $outputEncoding = NULL, $isXslString = false )
		{
			// 省略値を設定。2015.08.27 内部文字コードのUTF-8化に伴い、EUC-JPではなく基本的にUTF-8を指定するように変更。
			if ( is_null( $outputEncoding ) ) {
				$outputEncoding = gCMSDInternalEncoding(); // 'EUC-JP';
			}
		
			// XSLT-baseディレクトリを取得
			if ( ! $isXslString )
			{
				//$basePath = "file://" . dirname( $strXsl ) . '/';
				$basePath = dirname( $strXsl );
			}
			else
			{
				$basePath = '';
			}
		
			// XSLTライブラリ設定番号、XSLT文字コード変換抑制設定を取得
			$xsltLib = ADXslt::getXsltLibNo();
			$disableEncoding = ADXslt::getDisableXsltEncoding();
			
			//print "xsltLib = " . $xsltLib . ", disableXsltEncoding = " . ( $disableEncoding ? "True" : "False" );
			
			// XSLTライブラリによる文字コード変換を抑制する（指定アリの場合のみ）
			if ( $disableEncoding )
			{
				set_error_handler( 'ADXslt_error_handler2' );
				
				// XSLデータをDOMに読み込む
				if ( $isXslString )
				{
					$xmlDocXsl = @domxml_open_mem( $strXsl );
				}
				else
				{
					$xmlDocXsl = @domxml_open_file( $strXsl );
				}
				
				if ( !$xmlDocXsl )
				{
					print( "<dt>[XSL]</dt><dd>" . convencoding_xml2html( $strXsl ) . "</dd>\n" );
					exit();
				}
				restore_error_handler();
								
				// xsl:output/@encodingを取得する。
				$outputEncodingXsl = '';
				$xpathXsl = $xmlDocXsl->xpath_new_context();
				xpath_register_ns( $xpathXsl, "xsl", "http://www.w3.org/1999/XSL/Transform" );	// 名前空間を登録する。
				$xmlXslOutputArr = $xpathXsl->xpath_eval( "//*[self::xsl:output]" );	// xsl:outputを取得。'//xsl:output'だと、XREAサーバーのPHP5.3環境でハングアップする為、この表記に変えた。なぜハングアップするかは不明。namespaceに関係しているライブラリのバグの可能性が高い。
				if ( count( cmsd_get_nodeset( $xmlXslOutputArr ) ) > 0 )
				{
					// encoding属性を取得。存在するならば、'UTF-8'に差し替える。
					$outputEncodingXsl = $xmlXslOutputArr->nodeset[0]->get_attribute( 'encoding' );
					if ( strlen( $outputEncodingXsl ) > 0 )
					{
						$xmlXslOutputArr->nodeset[0]->set_attribute( 'encoding', 'UTF-8' );	// 固定でUTF-8を指定する。もしXSLTライブラリがこれを無視しても、ライブラリの規定文字コードがUTF-8なので問題ない。
					}
					
					// encoding属性が存在しない場合、XSLTライブラリは規定でUTF-8で出力するので問題ない。
				}
				
				if ( strlen( $outputEncodingXsl ) <= 0 )
				{
					//$outputEncodingXsl = 'EUC-JP';	// XSLT側でoutput encodingが指定されていなかった場合、EUC-JPが指定されたとみなす。
					$outputEncodingXsl = gCMSDInternalEncoding();	// 2015.08.27 UTF-8化対応
				}
				
				// XSLデータを差し替える
				$isXslString = true;
				$strXsl = $xmlDocXsl->dump_mem( true, 'UTF-8' );

			}
			
			
			$resultStr = '';
			
			switch ( $xsltLib )
			{
				case 10:	// Sabloton
					$strXml = $xmlDoc->dump_mem( true, 'UTF-8' );
					$strXml = utf8_replace_irregular_chars( $strXml );	// 2007.10.2 eucJP-win対応時の副作用（「潤オ」の件）を回避するコード。
					
					$xslt = xslt_create();
					
					xslt_set_error_handler( $xslt, 'ADXslt_error_handler' );
					
					// XSLT-baseディレクトリを強制的に設定する。
					if ( function_exists( 'xslt_set_base' ) and strlen( $basePath ) > 0 )
					{
						xslt_set_base( $xslt, 'file://'.$basePath.'/' );
					}
					
					if ( $isXslString )
					{
						// strXsl:XSL文字列の場合
						//print( $strXsl );
						$resultStr = @xslt_process( $xslt, 'arg:/_xml', 'arg:/_xsl', NULL, array( '/_xml' => $strXml, '/_xsl' => $strXsl ) );
					}
					else
					{
						// strXsl:ファイルパスの場合
						$resultStr = @xslt_process( $xslt, 'arg:/_xml', 'file://'.$strXsl, NULL, array( '/_xml' => $strXml ) );
					}
					
					/*
					// XSLTエラー発生
					if ( !$resultStr && xslt_errno( $xslt ) > 0 )
					{
						$errorMessage = "XSL Translation Error.<br />\n" .
										"- Message: " . xslt_error( $xslt ) . "<br />\n" .
										"- XSL:  " .
										"<pre>" . convencoding_xml2html( $strXsl ) . "</pre>";
						cmsd_abort_proc( $errorMessage );
					}
					*/
					xslt_free( $xslt );
				
					break;
				
				case 20:	// domxml
				case 21:	// domxml(古い関数を使用）
					set_error_handler( 'ADXslt_error_handler2' );
				
					if ( $isXslString )
					{
						// strXsl:XSL文字列の場合
						$xslt = domxml_xslt_stylesheet( $strXsl );
					}
					else
					{
						// strXsl:ファイルパスの場合
						$xslt = domxml_xslt_stylesheet_file( $strXsl );
					}
					
					if ( !$xslt )
					{
						print( "<dt>[XSL]</dt><dd>" . convencoding_xml2html( $strXsl ) . "</dd>\n" );
						exit();
					}
					
					// XSLT-baseディレクトリを強制的に設定する。
					if ( strlen( $basePath ) > 0 )
					{
						// xslt_set_base は使えないので、chdir で移動する。
						$currdir = getcwd();
						chdir( $basePath );
					}
					
					// 2007.10.2 eucJP-win対応時の副作用（「潤オ」の件）を回避するコード。
					$strXml = $xmlDoc->dump_mem( true, 'UTF-8' );
					$strXml = utf8_replace_irregular_chars( $strXml );
					$xmlDoc = domxml_open_mem( $strXml );
					
					// XSLT
					$result = $xslt->process( $xmlDoc );
					
					if ( !$result )
					{
						print( "<dt>[XSL]</dt><dd>" . convencoding_xml2html( $strXsl ) . "</dd>\n" );
						exit();
					}
					
					restore_error_handler();
					
					if ( strlen( $basePath ) > 0 )
					{
						// 元のディレクトリに戻す。
						chdir( $currdir );
					}
					
					if ( $xsltLib == 20 )
					{
						$resultStr = $xslt->result_dump_mem($result);
					}
					elseif ( $xsltLib == 21 )
					{
						$resultStr = $result->dump_mem();// false );//, $outputEncoding );
						
					}
					
					break;
				
				default:
					cmsd_abort_proc( 'XSLT functions is not supported.' . $xsltLib );
			}
			
			// XSLTライブラリによる文字コード変換を抑制した場合、ここで本来の文字コードに変換する。
			if ( $disableEncoding )
			{
				if ( strcasecmp( $outputEncodingXsl, 'UTF-8' ) != 0 )
				{
					if ( strcasecmp( $outputEncodingXsl, 'SHIFT_JIS' ) == 0 )
					{
						$outputEncodingXsl = 'SJIS-win'.''; // SJIS-win
					}
					elseif ( strcasecmp( $outputEncodingXsl, 'EUC-JP' ) == 0 )
					{
						$outputEncodingXsl = 'eucJP-win'.''; // eucJP-win
					}
					$resultStr = mb_convert_encoding( $resultStr, $outputEncodingXsl, 'UTF-8' );
				}
			}
			
			// 空要素の置換。
			// <br/>や<hr/>などを<br />や<hr />のように変換する。（最初、xsl:output method="xml"の場合だけにしようと思ったが、xslの中身を見に行くコストの方が大きい為やめた。）
			$resultStr = preg_replace( '/([^ ])\/>/', '$1 />', $resultStr );
			
			// <div />となってしまった出力を<div></div>のようにする。
			$resultStr = preg_replace( '/<(div|td|th|li|span) \/>/', '<$1></$1>', $resultStr );	// <div />が<div ></div>にならないよう、先に属性なしのタグを変換する。正規表現一発で済む方法があったら教えてください。
			$resultStr = preg_replace( '/<(div|td|th|li|span) ([^>]*)\/>/', '<$1 $2></$1>', $resultStr );
			
			$resultStr = trim( $resultStr );	// 2006.11.15 追加：XSLT変換結果の前後に余計な改行が追加される為、強制的に前後の改行や空白を除去。
			
			return $resultStr;
			
		}
		
		
		/* -----------------------------------------------------------
		 * [クラスメソッド]
		 * XSLT変換を実行する。
		 * &$xmlDoc : 変換対象のDOMオブジェクト。
		 * $xslFilePath : XSLファイルのパス。
		 * $outputEncoding : 出力用のエンコーディング。通常はXSLT側で指定したものが使われるが、環境によっては無視されるので直接指定する。
		*/
		function processStrXsl( &$xmlDoc, $strXsl, $outputEncoding = NULL )
		{
			// 省略値を設定。2015.08.27 内部文字コードのUTF-8化に伴い、EUC-JPではなく基本的にUTF-8を指定するように変更。
			if ( is_null( $outputEncoding ) ) {
				$outputEncoding = gCMSDInternalEncoding(); // 'EUC-JP';
			}
			
			return ADXslt::process( $xmlDoc, $strXsl, $outputEncoding, true );
			
		}		
		
	}
	
	// disable-xslt-encoding="True" の際に、XSLTライブラリが自動的にxsl中のMETAタグのcharsetをUTF-8にしてしまうのを回避する。
	// 具体的には、METAタグを再度EUC-JPに戻す。
	function replaceMetaTag( $source, $fromCharset, $toCharset )
	{
		// disable-xslt-encoding="True" の際に、XSLTライブラリが自動的にxsl中のMETAタグのcharsetをUTF-8にしてしまうのを回避する。
		// 具体的には、METAタグを再度EUC-JPに戻す。
		
		if ( strcasecmp( $fromCharset, $toCharset ) != 0 ) { // 2015.08.27 UTF-8化対応に伴い、UTF-8からUTF-8への変更に意味がない為、その場合は処理をパスする。
		
			$source = str_replace( array( 'charset='. strtolower( $fromCharset ) . '"', 'charset=' . strtoupper( $fromCharset ) . '"' ), 
										array( 'charset='. strtolower( $toCharset ) . '"', 'charset=' . strtoupper( $toCharset ) . '"' ), 
										$source );
		
		}
		return $source;
	}
	
	
?>